* RATS code to estimate VAR and calculate nonorthgonal IRF
* written by James Hamilton March 2016
cal 1954 3 4
      * identifies as quarterly observations beginning 1954:Q3
compute capt = 2015:4
      * a variable capt is introduced to facilitate updating with later data
open data var3_data.csv
data(org=obs) 1954:3 capt datex gdplev pce_def fedfunds
* col 1:  date
* col 2:  level of real GDP (2009 dollars)
* col 3:  level of implicit price deflator on personal consumption expenditures (2009 = 100)
* col 4:  average value of effective fed funds rate over the quarter

* convert GDP and price level to annual growth rates
set gdpch = 400*log(gdplev(t)/gdplev(t-1))
set inflation = 400*log(pce_def(t)/pce_def(t-1))

* set NBER bus cycle dates if desired
set recessdum = 0
* set recessdum 1948:4 1949:4 = 1
* set recessdum 1953:2 1954:2 = 1
set recessdum 1957:3 1958:2 = 1
set recessdum 1960:2 1961:1 = 1
set recessdum 1969:4 1970:4 = 1
set recessdum 1973:4 1975:1 = 1
set recessdum 1980:1 1980:3 = 1
set recessdum 1981:3 1982:4 = 1
set recessdum 1990:3 1991:1 = 1
set recessdum 2001:1 2001:4 = 1
set recessdum 2007:4 2009:2 = 1
open styles darker.txt

* plot data
spgraph(vfields=3)
smpl 1954:3 capt
graph(dates,header='GDP growth',shading=recessdum) 1
# gdpch
graph(dates,header='Inflation',shading=recessdum) 1
# inflation
graph(dates,header='Fed funds rate',shading=recessdum) 1
# fedfunds
spgraph(done)

system
variables gdpch inflation fedfunds
lags 1 to 5
det constant
end(system)
estimate(noprint)
compute logdet5 = %logdet

* ========================================================================================
* test lag length

system(model=var4)
variables gdpch inflation fedfunds
lags 1 to 4
det constant
end(system)

smpl 1960:1 1990:4
estimate(sigma)
compute logdet4 = %logdet
compute tee = %nobs

compute test_chi = tee*(logdet4 - logdet5)
cdf chisqr test_chi 16

compute test_sims = (tee - 16.0)*(logdet4 - logdet5)
cdf chisqr test_sims 16

compute aic4 = logdet4 + 2*4.0*16/tee
compute aic5 = logdet5 + 2*5.0*16/tee
compute sic4 = logdet4 + (4.0*16/tee)*log(tee)
compute sic5 = logdet5 + (5.0*16/tee)*log(tee)

display 'aic for lags 4 5' aic4 aic5
disply 'schwarz for lags 4 5' sic4 sic5

* ===========================================================================================
* calculate nonorthgonalized IRF for VAR(4)

compute neqn = 3
compute nsteps = 20

declare rect[series] impblk(neqn,neqn)
declare vect[labels] implabel(neqn)
compute implabel=|| 'GDP', 'inflation', 'fed funds'||

smpl 1 nsteps
impulse(result=impblk,model=var4) neqn nsteps * %identity(neqn)
do j=1,neqn
spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Response of ' +implabel(i) +' to ' +implabel(j)
graph(header=graphlabel,nodates,number=0) 1
# impblk(i,j)
end do
spgraph(done)
end do

* ==============================================================================================
* calculate Jorda local projections
declare rect[series] impjorda(neqn,neqn)

do k=1,nsteps
smpl 1960:1 1990:4
linreg gdpch
# gdpch{k-1} inflation{k-1} fedfunds{k-1} constant gdpch{k to k+2} inflation{k to k+2} fedfunds{k to k+2}
smpl 1 nsteps
set impjorda(1,1) k k = %beta(1)
set impjorda(1,2) k k = %beta(2)
set impjorda(1,3) k k = %beta(3)
end do
spgraph(vfields=neqn)
do j=1,neqn
compute graphlabel='Response of GDP to ' +implabel(j) +' (local projection)'
graph(header=graphlabel,nodates,number=0)
# impjorda(1,j)
end do
spgraph(done)

spgraph(vfields=neqn)
do j=1,neqn
compute graphlabel='Response of GDP to ' +implabel(j) +' (VAR)'
graph(header=graphlabel,nodates,number=0)
# impblk(1,j)
end do
spgraph(done)

print(nodates) 1 nsteps impblk(1,1) impjorda(1,1) impblk(1,3) impjorda(1,3)

* ======================================================================
* now try it in levels

smpl 1954:3 capt
set gdplog = 400*log(gdplev(t))
smpl 1960:1 1990:4
system(model=var4)
variables gdplog inflation fedfunds
lags 1 to 4
det constant
end(system)
estimate
smpl 1 nsteps
impulse(result=impblk,model=var4) neqn nsteps * %identity(neqn)
do j=1,neqn
spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Response of ' +implabel(i) +' to ' +implabel(j)
graph(header=graphlabel,nodates,number=0) 1
# impblk(i,j)
end do
spgraph(done)
end do

* ===========================================================================
* use later sample
smpl 1991:1 2007:4
system(model=var4)
variables gdpch inflation fedfunds
lags 1 to 4
det constant
end(system)
estimate
smpl 1 nsteps
impulse(result=impblk,model=var4) neqn nsteps * %identity(neqn)
do j=1,neqn
spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Response of ' +implabel(i) +' to ' +implabel(j) + '(1991:1-2007:4)'
graph(header=graphlabel,nodates,number=0) 1
# impblk(i,j)
end do
spgraph(done)
end do

*==================================================================================
*use full sample
smpl 1954:3 2007:4
system(model=var4)
variables gdpch inflation fedfunds
lags 1 to 4
det constant
end(system)
estimate
smpl 1 nsteps
impulse(result=impblk,model=var4) neqn nsteps * %identity(neqn)
do j=1,neqn
spgraph(vfields=neqn)
do i=1,neqn
compute graphlabel='Response of ' +implabel(i) +' to ' +implabel(j) + '(1954:3-2007:4)'
graph(header=graphlabel,nodates,number=0) 1
# impblk(i,j)
end do
spgraph(done)
end do
